@@ -11,7 +11,7 @@ from kilosort.gui import (
     DataConversionBox
 )
 from kilosort.gui.logger import setup_logger
-from kilosort.io import BinaryFiltered
+from kilosort.io import BinaryFiltered, remove_bad_channels
 from kilosort.utils import DOWNLOADS_DIR, download_probes
 from qtpy import QtCore, QtGui, QtWidgets
 
@@ -55,7 +55,11 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
         if self.qt_settings.contains('auto_load'):
             auto_load = self.qt_settings.value('auto_load')
-            if auto_load.lower() == 'false':
+            # Check for str and bool, seems like cache can store different types
+            # depending on Qt version or OS.
+            if isinstance(auto_load, str) and auto_load.lower() == 'false':
+                self.auto_load = False
+            elif isinstance(auto_load, bool) and auto_load is False:
                 self.auto_load = False
             else:
                 self.auto_load = True
@@ -64,7 +68,9 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
         if self.qt_settings.contains('show_plots'):
             show_plots = self.qt_settings.value('show_plots')
-            if show_plots.lower() == 'false':
+            if isinstance(show_plots, str) and show_plots.lower() == 'false':
+                self.show_plots = False
+            elif isinstance(show_plots, bool) and show_plots is False:
                 self.show_plots = False
             else:
                 self.show_plots = True
@@ -85,16 +91,17 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
         self.content = QtWidgets.QWidget(self)
         self.content_layout = QtWidgets.QVBoxLayout()
+        self.left_boxes = QtWidgets.QWidget()
+        self.left_boxes_layout = QtWidgets.QVBoxLayout(self.left_boxes)
+        self.right_boxes = QtWidgets.QWidget()
+        self.right_boxes_layout = QtWidgets.QVBoxLayout(self.right_boxes)
+        self.left_right_splitter = QtWidgets.QSplitter(QtCore.Qt.Horizontal)
+        self.settings_probe_splitter = QtWidgets.QSplitter(QtCore.Qt.Horizontal)
+        self.settings_message_splitter = QtWidgets.QSplitter(QtCore.Qt.Vertical)
+        self.settings_area = QtWidgets.QWidget()
+        self.settings_layout = QtWidgets.QVBoxLayout(self.settings_area)
 
         self.header_box = HeaderBox(self)
-
-        self.boxes = QtWidgets.QWidget()
-        self.boxes_layout = QtWidgets.QHBoxLayout(self.boxes)
-        self.first_boxes_layout = QtWidgets.QVBoxLayout()
-        self.second_boxes_layout = QtWidgets.QHBoxLayout()
-        self.third_boxes_layout = QtWidgets.QVBoxLayout()
-        self.fourth_boxes_layout = QtWidgets.QVBoxLayout()
-
         self.converter = DataConversionBox(self)
         self.settings_box = SettingsBox(self)
         self.probe_view_box = ProbeViewBox(self)
@@ -202,30 +209,50 @@ class KiloSortGUI(QtWidgets.QMainWindow):
     def setup(self):
         self.setWindowTitle(f"Kilosort4")
 
-        self.third_boxes_layout.addWidget(self.settings_box, 85)
-        self.third_boxes_layout.addWidget(self.run_box, 15)
-
-        self.second_boxes_layout.addLayout(self.third_boxes_layout, 50)
-        self.second_boxes_layout.addWidget(self.probe_view_box, 50)
-
-        self.first_boxes_layout.addLayout(self.second_boxes_layout, 60)
-        self.first_boxes_layout.addWidget(self.message_log_box, 40)
-
-        self.fourth_boxes_layout.addWidget(self.header_box, 3)
-        self.fourth_boxes_layout.addWidget(self.data_view_box, 97)
-
-        self.boxes_layout.addLayout(self.first_boxes_layout, 25)
-        self.boxes_layout.addLayout(self.fourth_boxes_layout, 75)
-
-        self.boxes.setLayout(self.boxes_layout)
-        self.content_layout.addWidget(self.boxes, 90)
+        # Set Wiget positions
+        # Top left, settings plus "load" and "run" buttons.
+        self.settings_layout.addWidget(self.settings_box, 85)
+        self.settings_layout.addWidget(self.run_box, 15)
+        # To right of settings, add probe view
+        self.settings_probe_splitter.addWidget(self.settings_area)
+        self.settings_probe_splitter.addWidget(self.probe_view_box)
+        # Below settings & probe, add message box
+        self.settings_message_splitter.addWidget(self.settings_probe_splitter)
+        self.settings_message_splitter.addWidget(self.message_log_box)
+        self.left_boxes_layout.addWidget(self.settings_message_splitter)
+        # Right-hand side, header plus data view
+        self.right_boxes_layout.addWidget(self.header_box, 3)
+        self.right_boxes_layout.addWidget(self.data_view_box, 97)
+        # Nest left and right within vertical splitter
+        self.left_right_splitter.addWidget(self.left_boxes)
+        self.left_right_splitter.addWidget(self.right_boxes)
 
+        self.content_layout.addWidget(self.left_right_splitter, 90)
         self.content_layout.setContentsMargins(10, 10, 10, 10)
         self.content.setLayout(self.content_layout)
         self.setCentralWidget(self.content)
 
-        self.header_box.reset_gui_button.clicked.connect(self.reset_gui)
+        # Restore splitter positions if saved from previous session
+        if self.qt_settings.contains('settings_probe_splitter'):
+            self.settings_probe_splitter.restoreState(
+                self.qt_settings.value('settings_probe_splitter')
+                )
+            self.settings_message_splitter.restoreState(
+                self.qt_settings.value('settings_message_splitter')
+                )
+            self.left_right_splitter.restoreState(
+                self.qt_settings.value('left_right_splitter')
+                )
+        else:
+            # Default to 25/75 split for left/right, 50/50 for settings/probe
+            # NOTE: The large values are used so that QT will divide the extra
+            #       pixels proportionally between the split widgets, which is
+            #       much simpler than figuring out the exact pixel positions.
+            self.left_right_splitter.setSizes([250000, 750000])
+            self.settings_probe_splitter.setSizes([500000, 500000])
 
+        # Connect signals
+        self.header_box.reset_gui_button.clicked.connect(self.reset_gui)
         self.settings_box.settingsUpdated.connect(self.load_data)
         self.settings_box.previewProbe.connect(self.probe_view_box.preview_probe)
         # Don't allow spike sorting to run until new data has actually
@@ -246,6 +273,20 @@ class KiloSortGUI(QtWidgets.QMainWindow):
         self.converter.disableInput.connect(self.disable_all_input)
         self.converter.fileObjectLoaded.connect(self.add_file_object)
 
+        self.settings_probe_splitter.splitterMoved.connect(self.save_widget_sizes)
+        self.settings_message_splitter.splitterMoved.connect(self.save_widget_sizes)
+        self.left_right_splitter.splitterMoved.connect(self.save_widget_sizes)
+
+    @QtCore.Slot()
+    def save_widget_sizes(self):
+        # Store splitter positions so that resizing persists between sessions.
+        state1 = self.settings_probe_splitter.saveState()
+        self.qt_settings.setValue('settings_probe_splitter', state1)
+        state2 = self.settings_message_splitter.saveState()
+        self.qt_settings.setValue('settings_message_splitter', state2)
+        state3 = self.left_right_splitter.saveState()
+        self.qt_settings.setValue('left_right_splitter', state3)
+
     def change_channel_display(self, direction):
         if self.context is not None:
             self.data_view_box.shift_primary_channel(direction)
@@ -289,14 +330,16 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
     def set_parameters(self):
         settings = self.settings_box.settings
+        bad_channels = self.settings_box.bad_channels
 
         self.data_path = settings["data_file_path"]
         self.results_directory = settings["results_dir"]
-        self.probe_layout = settings["probe"]
+        self.probe_layout = remove_bad_channels(settings["probe"], bad_channels)
         self.probe_name = settings["probe_name"]
         self.num_channels = settings["n_chan_bin"]
 
         params = settings.copy()
+        params['save_preprocessed_copy'] = self.run_box.save_preproc_check.isChecked()
 
         assert params
 
@@ -322,6 +365,7 @@ class KiloSortGUI(QtWidgets.QMainWindow):
     def load_binary_files(self):
         n_channels = self.params["n_chan_bin"]
         sample_rate = self.params["fs"]
+        cutoff = self.params['highpass_cutoff']
         chan_map = self.probe_layout["chanMap"]
         xc = self.probe_layout["xc"]
         yc = self.probe_layout["yc"]
@@ -330,6 +374,8 @@ class KiloSortGUI(QtWidgets.QMainWindow):
         tmin = self.params['tmin']
         tmax = self.params['tmax']
         artifact = self.params['artifact_threshold']
+        shift = self.params['shift']
+        scale = self.params['scale']
 
         if chan_map.max() >= n_channels:
             raise ValueError(
@@ -347,6 +393,8 @@ class KiloSortGUI(QtWidgets.QMainWindow):
             tmin=tmin,
             tmax=tmax,
             artifact_threshold=artifact,
+            shift=shift,
+            scale=scale,
             file_object=self.file_object
         )
 
@@ -354,6 +402,7 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
         self.context.highpass_filter = preprocessing.get_highpass_filter(
             fs=sample_rate,
+            cutoff=cutoff,
             device=self.device
         )
 
@@ -368,6 +417,8 @@ class KiloSortGUI(QtWidgets.QMainWindow):
             tmin=tmin,
             tmax=tmax,
             artifact_threshold=artifact,
+            shift=shift,
+            scale=scale,
             file_object=self.file_object
         ) as bin_file:
             self.context.whitening_matrix = preprocessing.get_whitening_matrix(
@@ -389,6 +440,8 @@ class KiloSortGUI(QtWidgets.QMainWindow):
             tmin=tmin,
             tmax=tmax,
             artifact_threshold=artifact,
+            shift=shift,
+            scale=scale,
             file_object=self.file_object
         )
 
@@ -433,7 +486,7 @@ class KiloSortGUI(QtWidgets.QMainWindow):
 
     @QtCore.Slot()
     def update_probe_view(self):
-        self.probe_view_box.set_layout(self.context)
+        self.probe_view_box.set_layout()
 
     def update_data_view(self):
         self.data_view_box.set_whitening_matrix(self.context.whitening_matrix)
@@ -505,6 +558,7 @@ class KiloSortGUI(QtWidgets.QMainWindow):
         self.message_log_box.reset()
 
     def closeEvent(self, event: QtGui.QCloseEvent):
+        # Make sure all threads and pop-out windows are closed as well.
         self.message_log_box.save_log_file()
         self.message_log_box.popout_window.close()
         for _, p in self.run_box.plots.items():
@@ -513,6 +567,7 @@ class KiloSortGUI(QtWidgets.QMainWindow):
         if self.converter.conversion_thread is not None:
             self.converter.conversion_thread.terminate()
         self.close_binary_files()
+
         event.accept()
 
 
