@@ -2,6 +2,7 @@ import os
 import pprint
 from pathlib import Path
 import json
+import ast
 
 import numpy as np
 import torch
@@ -21,7 +22,7 @@ _ALLOWED_FILE_TYPES = ['.bin', '.dat', '.bat', '.raw']  # For binary data
 
 class SettingsBox(QtWidgets.QGroupBox):
     settingsUpdated = QtCore.Signal()
-    previewProbe = QtCore.Signal(object, object)
+    previewProbe = QtCore.Signal(object)
     dataChanged = QtCore.Signal()
 
     def __init__(self, parent):
@@ -64,6 +65,19 @@ class SettingsBox(QtWidgets.QGroupBox):
         self._probes = []
         self.populate_probe_selector()
 
+        self.bad_channels_text = QtWidgets.QLabel("Excluded channels:")
+        self.bad_channels_input = QtWidgets.QLineEdit()
+        if self.gui.qt_settings.contains('bad_channels'):
+            bad_channels = self.gui.qt_settings.value('bad_channels')
+            if bad_channels is not None:
+                # List of ints gets cached as list of strings, so have to convert.
+                self.bad_channels = [int(s) for s in bad_channels]
+                self.bad_channels_input.setText(str(self.bad_channels))
+            else:
+                self.bad_channels = []
+        else:
+            self.bad_channels = []
+
         self.dtype_selector_text = QtWidgets.QLabel("Data dtype:")
         self.dtype_selector = QtWidgets.QComboBox()
         self.populate_dtype_selector()
@@ -139,6 +153,8 @@ class SettingsBox(QtWidgets.QGroupBox):
         self.load_settings_button.clicked.connect(self.update_settings)
         layout.addWidget(self.load_settings_button, row_count, col1, rspan, dbl)
 
+
+        ### Data selection / conversion
         row_count += rspan
         layout.addWidget(self.select_data_file, row_count, col1, rspan, cspan1)
         layout.addWidget(self.convert_data_button, row_count, col2, rspan, cspan2)
@@ -150,11 +166,13 @@ class SettingsBox(QtWidgets.QGroupBox):
             self.on_data_file_path_changed
         )
 
+
         # Add small vertical space for visual grouping
         row_count += rspan
         layout.addWidget(QtWidgets.QWidget(), row_count, 0, 1, dbl)
         row_count += 1
 
+        ### Results path
         layout.addWidget(
             self.select_results_directory, row_count, col1, rspan, cspan1
             )
@@ -169,11 +187,13 @@ class SettingsBox(QtWidgets.QGroupBox):
             self.on_results_directory_changed
         )
 
+
         # Add small vertical space for visual grouping
         row_count += rspan
         layout.addWidget(QtWidgets.QWidget(), row_count, 0, 1, dbl)
         row_count += 1
 
+        ### Probe selection
         layout.addWidget(self.probe_layout_text, row_count, col1, rspan, cspan1)
         layout.addWidget(
             self.probe_preview_button, row_count, col2, rspan, cspan2)
@@ -182,23 +202,29 @@ class SettingsBox(QtWidgets.QGroupBox):
 
         row_count += rspan
         layout.addWidget(self.probe_layout_selector, row_count, col1, rspan, dbl)
-        #self.probe_layout_selector.setSizeAdjustPolicy(
-        #    QtWidgets.QComboBox.AdjustToMinimumContentsLength
-        #)
         self.probe_layout_selector.currentTextChanged.connect(
             self.on_probe_layout_selected
         )
 
+        row_count += rspan
+        layout.addWidget(self.bad_channels_text, row_count, col1, rspan, cspan1)
+        layout.addWidget(self.bad_channels_input, row_count, col2, rspan, cspan2)
+        self.bad_channels_input.editingFinished.connect(self.update_bad_channels)
+        self.bad_channels_text.setToolTip(
+            "A list of channel indices (rows in the binary file) that should "
+            "not be included in sorting.\nListing channels here is equivalent to "
+            "excluding them from the probe dictionary."
+            )
+
+
         # Add small vertical space for visual grouping
         row_count += rspan
         layout.addWidget(QtWidgets.QWidget(), row_count, 0, 1, dbl)
         row_count += 1
 
+        ### Settings
         layout.addWidget(self.dtype_selector_text, row_count, col1, rspan, cspan1)
         layout.addWidget(self.dtype_selector, row_count, col2, rspan, cspan2)
-        #self.dtype_selector.setSizeAdjustPolicy(
-        #    QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLength
-        #)
         self.dtype_selector.currentTextChanged.connect(
             self.on_data_dtype_selected
         )
@@ -206,9 +232,6 @@ class SettingsBox(QtWidgets.QGroupBox):
         row_count += rspan
         layout.addWidget(self.device_selector_text, row_count, col1, rspan, cspan1)
         layout.addWidget(self.device_selector, row_count, col2, rspan, cspan2)
-        #self.device_selector.setSizeAdjustPolicy(
-        #    QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLength
-        #)
         self.device_selector.currentTextChanged.connect(
             self.on_device_selected
         )
@@ -242,6 +265,7 @@ class SettingsBox(QtWidgets.QGroupBox):
             )
         self.export_settings_button.clicked.connect(self.export_settings)
 
+
         self.setLayout(layout)
         self.set_cached_field_values()
         self.update_settings()
@@ -274,7 +298,13 @@ class SettingsBox(QtWidgets.QGroupBox):
         for k, p in EXTRA_PARAMETERS.items():
             if self.gui.qt_settings.contains(k):
                 # Use cached value
-                d = str(self.gui.qt_settings.value(k))
+                v = self.gui.qt_settings.value(k)
+                if k == 'drift_smoothing':
+                    # List of floats gets cached as list of strings, so
+                    # have to convert back.
+                    d = str([float(s) for s in v])
+                else:
+                    d = str(v)
             else:
                 # Use default value
                 d = str(p['default'])
@@ -429,13 +459,18 @@ class SettingsBox(QtWidgets.QGroupBox):
 
     def check_valid_binary_path(self, filename):
         if filename is None:
+            print('Binary path is None.')
             return False
         else:
             f = Path(filename)
             if f.exists() and f.is_file():
                 if f.suffix in _ALLOWED_FILE_TYPES or self.use_file_object:
                     return True
+                else:
+                    print(f'Binary file has invalid suffix. Must be {_ALLOWED_FILE_TYPES}')
+                    return False
             else:
+                print('Binary file does not exist at that path.')
                 return False
 
     def disable_all_input(self, value):
@@ -474,7 +509,10 @@ class SettingsBox(QtWidgets.QGroupBox):
         if not self.check_valid_binary_path(self.data_file_path):
             return False
 
-        none_allowed = ['dmin', 'nt0min', 'max_channel_distance', 'x_centers']
+        none_allowed = [
+            'dmin', 'nt0min', 'max_channel_distance', 'x_centers',
+            'shift', 'scale'
+            ]
         for k, v in self.settings.items():
             if v is None and k not in none_allowed:
                 print(f'`None` not allowed for parameter {k}.')
@@ -511,7 +549,7 @@ class SettingsBox(QtWidgets.QGroupBox):
 
     @QtCore.Slot()
     def show_probe_layout(self):
-        self.previewProbe.emit(self.probe_layout, self.get_probe_template_args())
+        self.previewProbe.emit(self.get_probe_template_args())
 
     @QtCore.Slot(str)
     def on_probe_layout_selected(self, name):
@@ -638,6 +676,32 @@ class SettingsBox(QtWidgets.QGroupBox):
         self.gui.qt_settings.setValue('probe_layout', layout)
         self.gui.qt_settings.setValue('probe_name', name)
 
+    def get_bad_channels(self):
+        text = self.bad_channels_input.text()
+        text = text.replace(']','').replace('[','').replace(' ','')
+        if len(text) > 0:
+            bad_channels = [int(s) for s in text.split(',')]
+        else:
+            bad_channels = []
+        
+        return bad_channels
+
+    def set_bad_channels(self, bad_channels):
+        self.bad_channels_input.setText(str(bad_channels))
+        self.bad_channels_input.editingFinished.emit()
+
+    @QtCore.Slot()
+    def update_bad_channels(self):
+        # Remove brackets and white space if present, convert to list of ints.
+        self.bad_channels = self.get_bad_channels()
+        self.gui.qt_settings.setValue('bad_channels', self.bad_channels)
+
+        # Trigger update so that probe layout in main gets updated, then
+        # refresh probe view.
+        self.update_settings()
+        self.previewProbe.emit(self.get_probe_template_args)
+
+
     def on_data_dtype_selected(self, data_dtype):
         self.data_dtype = data_dtype
         self.gui.qt_settings.setValue('data_dtype', data_dtype)
@@ -808,7 +872,9 @@ def _check_parameter(sender_obj, main_obj, k, p):
             v = None
         else:
             v = _str_to_type(value, p['type'])
-            if not isinstance(v, bool):
+            if isinstance(v, bool) or isinstance(v, list):
+                pass
+            else:
                 assert v >= p['min']
                 assert v <= p['max']
                 assert v not in p['exclude']
@@ -848,6 +914,8 @@ def _str_to_type(string, dtype):
             v = True
         else:
             raise TypeError(f'{string} should be True or False for bool.')
+    elif dtype is list:
+        v = ast.literal_eval(string)
     else:
         v = dtype(string)
     return v
